#!/usr/bin/python3
import os, sys, re
import frontmatter #python-frontmatter
from datetime import datetime, date

consts = {
	'me_fedi': 'https://mstdn.social/@anedroid'
}
proxies = {
	'youtube.com': 'vid.puffyan.us',
	'youtu.be': 'vid.puffyan.us',
	'reddit.com': 'teddit.net',
	'twitter.com': 'nitter.net'
}

os.chdir(sys.path[0])

if len(sys.argv) < 2:
	print('preset (directory name) not specified')
	exit()
preset = sys.argv[1]
if not os.path.isdir(f'{preset}/_src'):
	print('specified preset does not exist')
	exit()
os.chdir(preset)

index_file = f'../{preset}.gmi'

with open('_index.template') as file:
	template = file.read()
articles = []

for file in os.scandir('_src'):
	if file.is_file() and file.path.endswith('.gmi'):
		print(f'Parsing {file.name}')
		post = frontmatter.load(file.path)
		post['name'] = os.path.basename(file.path)
		post['no_time'] = type(post['created']) == date
		post['created'] = datetime.fromisoformat(str(post['created']))
		articles.append(post)

articles.sort(key=lambda post:
	post['created'] if 'created' in post else datetime.fromtimestamp(0),
	reverse=True)

def parse_content(content):
	content = re.sub(r'(?<=\n=>)(\s+)([^\s]*)', parse_urls, content)
	content = re.sub(r'{([^}\n]*)}', parse_control, content)
	return content

def parse_urls(url):
	white, url = url.groups()
	return white+router(url)

def router(url):
	if url.startswith('http'):
		exp = re.compile(r'https?://([-\w.]+)/?(.*)?')
		match = exp.match(url)
		domain, path = match.groups()
		if domain in proxies:
			url = f'https://{proxies[domain]}/{path}'
	return url

def parse_control(match):
	match = match.group(1)
	var = re.match(r'\$(\w+)', match)
	func = re.match(r'(\w+)\(([^)]*)\)', match)
	if var:
		name = var.group(1)
		return consts[name]
	else:
		name, value = func.groups()
		if name == 'log':
			return f'../gemlog/{value}.gmi'
		elif name == 'article':
			return f'../articles/{value}.gmi'
		elif name == 'note':
		    return f'../sidenotes/{value}.gmi'
		else:
			print(f'unknown function {name}')

index = ''
for post in articles:
	index += f'\n=> {preset}/{post["name"]} '
	content = ''
	if 'created' in post:
		if post['no_time']:
			created_str = str(post['created'].date())
		else:
			created_str = str(post['created'])
		index += f'{post["created"].date()} '
		content += f'{created_str}\n'
	index += post['title']
	if 'description' in post:
		index += f'\n{post["description"]}'
	if 'author' in post:
		content += f'{post["author"]}\n'
	if 'source' in post:
		source = router(post['source'])
		content += f'=> {source} oryginalny artykuł\n'
	content += f'\n# {post["title"]}\n'
	content += '\n' + parse_content(str(post))
	print(content, file=open(post['name'], 'w'))
	print(template.replace('---index---', index), file=open(index_file, 'w'))